@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 #
 # Copyright 2013-2017 Red Hat, Inc
 # Copyright 2012-2015 eNovance <licensing@enovance.com>
@@ -36,13 +35,11 @@ profiler_opts = importutils.try_import('osprofiler.opts')
 OPTS = [
     cfg.IntOpt('http_timeout',
                default=600,
+               deprecated_for_removal=True,
+               deprecated_reason=('This parameter is not used now and has no '
+                                  'effect.'),
                help='Timeout seconds for HTTP requests. Set it to None to '
                     'disable timeout.'),
-    cfg.IntOpt('evaluation_interval',
-               default=60,
-               help='Period of evaluation cycle, should'
-               ' be >= than configured pipeline interval for'
-               ' collection of underlying meters.'),
 ]
 
 EVALUATOR_OPTS = [
@@ -50,7 +47,13 @@ EVALUATOR_OPTS = [
                default=1,
                min=1,
                help='Number of workers for evaluator service. '
-               'default value is 1.')
+               'default value is 1.'),
+    cfg.IntOpt('evaluation_interval',
+               default=60,
+               deprecated_group='DEFAULT',
+               help='Period of evaluation cycle, should'
+               ' be >= than configured pipeline interval for'
+               ' collection of underlying meters.'),
 ]
 
 NOTIFIER_OPTS = [
